open Mts;;

(* Global parameters. *)
let max_iter = 4096
and precision = 5
and threshold = 0.
and parameterP = 0.5
and formula = formulaAdd
;;

(* Main function. *)
let main fsp1 fsp2 spot =
  let pair1 = ref ("",{alphabet = [||];states=0;transitions=[||]})
  and pair2 = ref ("",{alphabet = [||];states=0;transitions=[||]})
  and chan1 = open_in fsp1
  and chan2 = open_in fsp2
  in
  try
    let lexbuf = Lexing.from_channel chan1
    in
      while true do
        let parsed = FspParser.main FspLexer.token lexbuf
	in
	  pair1 := toMts parsed
      done
  with FspLexer.Eof ->
    try
      let lexbuf = Lexing.from_channel chan2
      in
	while true do
          let parsed = FspParser.main FspLexer.token lexbuf
	  in
	    pair2 := toMts parsed
	done
    with FspLexer.Eof ->
      begin
	close_in chan1;
	close_in chan2;
	let name1,mts1temp = !pair1
	and name2,mts2temp = !pair2
	in
	let alphaUnion = arrayUnion mts1temp.alphabet mts2temp.alphabet
	in
	let mts1 = extendAlphabet False mts1temp alphaUnion
	and mts2 = extendAlphabet False mts2temp alphaUnion
	in
	let simvals = normalise parameterP (iterate_lazy precision (formula parameterP) mts1 mts2 label_sim [] max_iter)
	in
	let consist = extractConsistency true threshold simvals
	in
	let pseudoMerge = pseudo_mergeMtsGen false "_1" "_2" mts1 mts2 consist []
	and marked = pseudo_mergeMtsGen true "_1" "_2" mts1 mts2 consist []
	and newname = String.capitalize (name1 ^ "__" ^ name2 ^ "__")
	in
	  printMtsFsp newname pseudoMerge;
	  print_string ("\n||PseudoMerge__" ^ name1 ^ "__" ^ name2 ^ " = (" ^ newname ^ ").\n");
	  match spot with
	    | None -> ()
	    | Some(spotted) ->
		begin
		  let buf = Lexing.from_string spotted
		  in
		  let o,l,d = FspParser.triplet FspLexer.token buf
		  in
		    let path = extractPath pseudoMerge true o d l
		    in
		    let mu = pathToMuNonDet marked pseudoMerge mts1 mts2 simvals path
		    in
		      print_string ("/*  Distinguishing mu-formula\n\t"^(muCalcToString mu)^"\n\t"^(muCalcToLatex mu)^"\n*/\n")
(*
		    let scenName = String.capitalize "Scen"
		    and propName = String.capitalize "Prop"
		    and cap1 = String.capitalize name1
		    and cap2 = String.capitalize name2
		    and p1s2,p2s1 =
		      let consist = "consistent"
		      and incons = "inconsistent"
		      in
			if (String.sub l ((String.length l) - 1) 1) = "1"
			then 
			  incons,consist
			else
			  consist,incons
		    in
		   (* let propMts = pathToMts pseudoMerge true path *)
		    let propMts = pathToMtsBis pseudoMerge true "_1" "_2" path
		    in
		      print_string "\n";
		     (* pathToTree scenName pseudoMerge true path; *)
		      pathToTreeBis scenName pseudoMerge true "_1" "_2" path;
		     (* printMtsFsp propName propMts; *)
		      pathToProperty "PropTest" pseudoMerge true path;
		      print_string ("\n||" ^ propName ^ scenName ^ " = (" ^ propName ^ " || " ^ scenName ^ ").\n\n" ^
				      "||TestP1 = (" ^ propName ^ " ++ " ^ cap1 ^ "). // Should be " ^ p1s2 ^ ".\n" ^
				      "||TestP2 = (" ^ propName ^ " ++ " ^ cap2 ^ "). // Should be " ^ p2s1 ^ ".\n" ^
				      "||TestS1 = (" ^ scenName ^ " ++ " ^ cap1 ^ "). // Should be " ^ p2s1 ^ ".\n" ^
				      "||TestS2 = (" ^ scenName ^ " ++ " ^ cap2 ^ "). // Should be " ^ p1s2 ^ ".\n"
				   );
*)

		end
      end
;;


let _ =
  let argc = Array.length Sys.argv
  in
    if argc < 3
    then
      print_string ("Usage:\n\t" ^ Sys.argv.(0) ^ " <mts1.trans> <mts2.trans>\nwhere the content of files mts*.trans is in the form of transitions as in the \"Transitions\" tab of MTSA.\n")
    else
	main Sys.argv.(1) Sys.argv.(2) (if argc > 3 then Some(Sys.argv.(3)) else None)
;;
